@@ -72,6 +72,8 @@ fn open_browser_default(target: &TargetType, options: &BrowserOptions) -> Result
 
             "wsl" => try_wsl(options, target),
 
+            "flatpak" => try_flatpak(options, target),
+
             _ => Err(r),
         })
         // at the end, we'll try x-www-browser and return the result as is
@@ -144,6 +146,14 @@ fn is_wsl() -> bool {
     }
 }
 
+/// Check if we're running inside Flatpak
+#[inline]
+fn is_flatpak() -> bool {
+    std::env::var("container")
+        .map(|x| x.eq_ignore_ascii_case("flatpak"))
+        .unwrap_or(false)
+}
+
 /// Detect the desktop environment
 fn guess_desktop_env() -> &'static str {
     let unknown = "unknown";
@@ -154,7 +164,9 @@ fn guess_desktop_env() -> &'static str {
         .unwrap_or_else(|_| unknown.into())
         .to_ascii_lowercase();
 
-    if xcd.contains("gnome") || xcd.contains("cinnamon") || dsession.contains("gnome") {
+    if is_flatpak() {
+        "flatpak"
+    } else if xcd.contains("gnome") || xcd.contains("cinnamon") || dsession.contains("gnome") {
         // GNOME and its derivatives
         "gnome"
     } else if xcd.contains("kde")
@@ -207,6 +219,22 @@ fn try_wsl(options: &BrowserOptions, target: &TargetType) -> Result<()> {
     }
 }
 
+/// Open browser in Flatpak environments
+fn try_flatpak(options: &BrowserOptions, target: &TargetType) -> Result<()> {
+    match target.0.scheme() {
+        "http" | "https" => {
+            let url: &str = target;
+            // we assume xdg-open to be present, given that it's a part of standard
+            // runtime & SDK of flatpak
+            try_browser!(options, "xdg-open", url)
+        }
+        // we support only http urls under Flatpak to adhere to the defined
+        // Consistent Behaviour, as effectively DBUS is used interally, and
+        // there doesn't seem to be a way for us to determine actual browser
+        _ => Err(Error::new(ErrorKind::NotFound, "only http urls supported")),
+    }
+}
+
 /// Handle Haiku explicitly, as it uses an "open" command, similar to macos
 /// but on other Unixes, open ends up translating to shell open fd
 fn try_haiku(options: &BrowserOptions, url: &str) -> Result<()> {
@@ -346,9 +374,15 @@ fn open_using_xdg_config(config_path: &PathBuf, options: &BrowserOptions, url: &
 fn get_xdg_dirs() -> Vec<PathBuf> {
     let mut xdg_dirs: Vec<PathBuf> = Vec::new();
 
-    if let Some(user_dir) = dirs::data_dir() {
-        xdg_dirs.push(user_dir);
+    let data_home = std::env::var("XDG_DATA_HOME")
+        .ok()
+        .map(PathBuf::from)
+        .filter(|path| path.is_absolute())
+        .or_else(|| home::home_dir().map(|path| path.join(".local/share")));
+    if let Some(data_home) = data_home {
+        xdg_dirs.push(data_home);
     }
+
     if let Ok(data_dirs) = std::env::var("XDG_DATA_DIRS") {
         for d in data_dirs.split(':') {
             xdg_dirs.push(PathBuf::from(d));
